//------------------------------------------------------------------------------
// Copyright (C) 2011, Robert Johansson, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>

#include <stdio.h>
#include <mbus.h>

#include "mbus-tcp.h"

//------------------------------------------------------------------------------
// send a data request packet to from master to slave
//------------------------------------------------------------------------------
int
send_ping_frame(mbus_tcp_handle *handle, int address)
{
    int retval = 0;
    mbus_frame *frame;
        
    frame = mbus_frame_new(MBUS_FRAME_TYPE_SHORT);
    frame->control  = MBUS_CONTROL_MASK_SND_NKE | MBUS_CONTROL_MASK_DIR_M2S;
    frame->address  = address; 
   
    if (mbus_tcp_send_frame(handle, frame) == -1)
    {
        fprintf(stderr, "%s: failed to send mbus frame\n", __PRETTY_FUNCTION__);
        retval = -1;
    }    
	
    mbus_frame_free(frame);
    return retval;
}

//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    mbus_tcp_handle *handle;
    char *host;
    int port, address;

    if (argc != 3)
    {
        printf("usage: %s host port\n", argv[0]);
        return 0;
    }
 
    host = argv[1];   
    port = atoi(argv[2]);
    
    if ((handle = mbus_tcp_connect(host, port)) == NULL)
    {
        printf("Scan failed: Could not setup connection to M-bus gateway: %s\n",  mbus_error_str());
        return -1;
    }

    for (address = 0; address < 254; address++)
    {
        mbus_frame reply;

        if (send_ping_frame(handle, address) == -1)
        {
            printf("Scan failed. Could not send ping frame: %s\n", mbus_error_str());
            return -1;
        } 

        if (mbus_tcp_recv_frame(handle, &reply) == -1)
        {
            continue;
        }    

        if (mbus_frame_type(&reply) == MBUS_FRAME_TYPE_ACK)
        {
            printf("Found a M-Bus device at address %d\n", address);
        }
    }

    mbus_tcp_disconnect(handle);
    return 0;
}

